@@ -5,6 +5,8 @@ import { ObservableMap } from '@jupyterlab/observables';
 
 import { ISerializers, WidgetModel } from '@jupyter-widgets/base';
 
+import { ArrayExt } from '@lumino/algorithm';
+
 import { CommandRegistry } from '@lumino/commands';
 
 import { ReadonlyPartialJSONObject } from '@lumino/coreutils';
@@ -26,6 +28,8 @@ export class CommandRegistryModel extends WidgetModel {
       _model_name: CommandRegistryModel.model_name,
       _model_module: CommandRegistryModel.model_module,
       _model_module_version: CommandRegistryModel.model_module_version,
+      _command_list: [],
+      _commands: [],
     };
   }
 
@@ -43,9 +47,13 @@ export class CommandRegistryModel extends WidgetModel {
       if (this.comm_live) {
         return;
       }
-      this._customCommands.values().forEach((command) => command.dispose());
+      Private.customCommands.values().forEach((command) => command.dispose());
       this._sendCommandList();
     });
+
+    // restore existing commands
+    const commands = this.get('_commands');
+    commands.forEach((command: any) => this._addCommand(command));
     this._sendCommandList();
   }
 
@@ -59,11 +67,16 @@ export class CommandRegistryModel extends WidgetModel {
       case 'execute':
         this._execute(msg.payload);
         break;
-      case 'addCommand':
-        void this._addCommand(msg.payload);
+      case 'addCommand': {
+        this._addCommand(msg.payload);
+        // keep track of the commands
+        const commands = this.get('_commands');
+        this.set('_commands', commands.concat(msg.payload));
+        this.save_changes();
         break;
+      }
       case 'removeCommand':
-        void this._removeCommand(msg.payload);
+        this._removeCommand(msg.payload);
         break;
       default:
         break;
@@ -75,7 +88,7 @@ export class CommandRegistryModel extends WidgetModel {
    */
   private _sendCommandList(): void {
     this._commands.notifyCommandChanged();
-    this.set('_commands', this._commands.listCommands());
+    this.set('_command_list', this._commands.listCommands());
     this.save_changes();
   }
 
@@ -97,27 +110,32 @@ export class CommandRegistryModel extends WidgetModel {
    *
    * @param options The command options.
    */
-  private async _addCommand(
+  private _addCommand(
     options: CommandRegistry.ICommandOptions & { id: string }
-  ): Promise<void> {
+  ): void {
     const { id, caption, label, iconClass } = options;
     if (this._commands.hasCommand(id)) {
-      // TODO: handle this?
-      return;
+      Private.customCommands.get(id).dispose();
     }
+
+    const commandEnabled = (command: IDisposable): boolean => {
+      return !command.isDisposed && !!this.comm && this.comm_live;
+    };
     const command = this._commands.addCommand(id, {
       caption,
       label,
       iconClass,
       execute: () => {
         if (!this.comm_live) {
-          console.log('TODO: dispose the command');
+          command.dispose();
           return;
         }
         this.send({ event: 'execute', id }, {});
       },
+      isEnabled: () => commandEnabled(command),
+      isVisible: () => commandEnabled(command),
     });
-    this._customCommands.set(id, command);
+    Private.customCommands.set(id, command);
     this._sendCommandList();
   }
 
@@ -128,9 +146,13 @@ export class CommandRegistryModel extends WidgetModel {
    */
   private _removeCommand(bundle: { id: string }): void {
     const { id } = bundle;
-    if (this._customCommands.has(id)) {
-      this._customCommands.get(id).dispose();
+    if (Private.customCommands.has(id)) {
+      Private.customCommands.get(id).dispose();
     }
+    const commands = this.get('_commands').slice();
+    ArrayExt.removeAllWhere(commands, (w: any) => w.id === id);
+    this.set('_commands', commands);
+    this.save_changes();
     this._sendCommandList();
   }
 
@@ -146,7 +168,13 @@ export class CommandRegistryModel extends WidgetModel {
   static view_module_version = MODULE_VERSION;
 
   private _commands: CommandRegistry;
-  private _customCommands = new ObservableMap<IDisposable>();
 
   static commands: CommandRegistry;
 }
+
+/**
+ * A namespace for private data
+ */
+namespace Private {
+  export const customCommands = new ObservableMap<IDisposable>();
+}
